package id

import (
	"testing"

	jsoniter "github.com/json-iterator/go"
	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

func TestID_MarshalUnmarshalJSON(t *testing.T) {
	tests := []struct {
		name string
		id   *ObjectId
	}{
		{
			name: "OrganizationID",
			id:   &ObjectId{Descriptor: &OrganizationId{OrganizationID: "1"}},
		},
		{
			name: "UserID",
			id:   &ObjectId{Descriptor: &UserId{UserID: "1"}},
		},
		{
			name: "ServiceID",
			id:   &ObjectId{Descriptor: &ServiceId{ServiceID: "1"}},
		},
		{
			name: "SpaceId",
			id:   &ObjectId{Descriptor: &SpaceId{SpaceID: "1"}},
		},
		{
			name: "EnvironmentID",
			id:   &ObjectId{Descriptor: &EnvironmentId{EnvironmentID: "1", SpaceId: SpaceId{SpaceID: "1"}}},
		},
		{
			name: "ClientID",
			id:   &ObjectId{Descriptor: &ClientId{ClientID: "1", SpaceId: SpaceId{SpaceID: "1"}}},
		},
		{
			name: "RoleID",
			id:   &ObjectId{Descriptor: &RoleId{RoleID: "1", SpaceId: SpaceId{SpaceID: "1"}}},
		},
		{
			name: "CollectionId",
			id:   &ObjectId{Descriptor: &CollectionId{CollectionID: "1", EnvironmentId: EnvironmentId{EnvironmentID: "1", SpaceId: SpaceId{SpaceID: "1"}}}},
		},
		{
			name: "SchemaID",
			id:   &ObjectId{Descriptor: &SchemaId{CollectionID: "1", EnvironmentId: EnvironmentId{EnvironmentID: "1", SpaceId: SpaceId{SpaceID: "1"}}}},
		},
		{
			name: "ItemId",
			id:   &ObjectId{Descriptor: &ItemId{ItemID: "1", CollectionId: CollectionId{CollectionID: "1", EnvironmentId: EnvironmentId{EnvironmentID: "1", SpaceId: SpaceId{SpaceID: "1"}}}}},
		},
		{
			name: "RevisionID",
			id:   &ObjectId{Descriptor: &RevisionId{RevisionID: "1", ItemId: ItemId{ItemID: "1", CollectionId: CollectionId{CollectionID: "1", EnvironmentId: EnvironmentId{EnvironmentID: "1", SpaceId: SpaceId{SpaceID: "1"}}}}}},
		},
		{
			name: "FieldId",
			id:   &ObjectId{Descriptor: &FieldId{Field: "1", ItemId: ItemId{ItemID: "1", CollectionId: CollectionId{CollectionID: "1", EnvironmentId: EnvironmentId{EnvironmentID: "1", SpaceId: SpaceId{SpaceID: "1"}}}}}},
		},
		{
			name: "SystemID",
			id:   &ObjectId{Descriptor: &SystemId{}},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			b, err := jsoniter.Marshal(&tt.id)
			require.NoError(t, err)

			var i ObjectId
			require.NoError(t, jsoniter.Unmarshal(b, &i))
			assert.Equal(t, tt.id, &i, "после Unmarshal объект должен быть идентичен исходному")
		})
	}
}

func TestID_ExampleJSON(t *testing.T) {
	type data struct {
		ID     *ObjectId
		Text   string
		Number int
	}

	test := &data{
		ID:     &ObjectId{Descriptor: &SpaceId{SpaceID: Space}},
		Text:   "text",
		Number: 1,
	}

	b, err := jsoniter.Marshal(test)
	require.NoError(t, err)

	buf := new(data)
	err = jsoniter.Unmarshal(b, &buf)
	require.NoError(t, err)
	assert.Equal(t, test, buf, "после Unmarshal объект должен совпадать с исходным")
}
