package schema

import (
	"io"

	jsoniter "github.com/json-iterator/go"
	"gopkg.in/yaml.v3"
)

func FromYAML(r io.Reader) (s *Schema, err error) {
	yml, err := io.ReadAll(r)
	if err != nil {
		return nil, err
	}

	s = New()
	err = s.UnmarshalYAML(yml)
	return s, err
}

func (s *Schema) UnmarshalYAML(b []byte) error {
	jsonData, err := yamlToJson(b)
	if err != nil {
		return err
	}

	return s.UnmarshalJSON(jsonData)
}

func (s *Schema) MarshalYAML() ([]byte, error) {
	jsonData, err := s.MarshalJSON()
	if err != nil {
		return nil, err
	}

	return jsonToYaml(jsonData)
}

func jsonToYaml(b []byte) ([]byte, error) {
	var data interface{}
	if err := jsoniter.Unmarshal(b, &data); err != nil {
		return nil, err
	}

	return yaml.Marshal(data)
}

func yamlToJson(b []byte) ([]byte, error) {
	var data interface{}
	if err := yaml.Unmarshal(b, &data); err != nil {
		return nil, err
	}

	return jsoniter.Marshal(data)
}
